/*
  Compare mean of boys and girls, using effect of age to generate the two means.
  For simplicity, the true mean and SD of age, and its effect, are the same for 
boys and girls.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
*b=boy, g=girl;
%let dep=VO2max; *the dependent variable;
%let meanb=380; *true grand mean of dependent, back transforms to exp(380/100)=44.7;
%let meang=370; *true grand mean of dependent, back transforms to exp(370/100)=40.4;
%let meanage=13; *true mean age of kids;
%let sdage=2; *true between-subject SD of age;
%let slope=2; *true effect of age (per year) on the dependent;
%let tee=5; *typical (standard) error of the estimate, representing the differences
     in &dep between kids of a given age;
%let ssizeb=100; *sample size;
%let ssizeg=100; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for effects ;
%let decpred=1; *decimal places for simple stats of predictor (age);
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
Sex="boy ";
do ChildID=1 to &ssizeb;
  Age=&meanage+&sdage*rannor(0);
  Obsvd&dep=&meanb+&slope*(Age-&meanage)+&tee*rannor(0);
  output;
  end;
Sex="girl";
do ChildID=&ssizeb+1 to &ssizeb+&ssizeg;
  Age=&meanage+&sdage*rannor(0);
  Obsvd&dep=&meang+&slope*(Age-&meanage)+&tee*rannor(0);
  output;
  end;

*generate age groups;
data dat2;
set dat1;
AgeGroup="1.prepub ";
if Age>11 then AgeGroup="2.peripub";
if Age>15 then AgeGroup="3.postpub";

title "Raw data, first 10 observations";
proc print data=dat2(obs=10);
format Age 5.&decpred Obsvd&dep 5.&decdep;
run;

title "Simple stats";
proc means maxdec=&decdep;
var Obsvd&dep Age;
class Sex;
*class Sex AgeGroup;
run;

ods graphics / reset width=10cm height=10cm imagemap attrpriority=none;
title "Means and SD";
proc sgplot data=dat2;
styleattrs  
  datacolors=(lightskyblue pink darksalmon yellow lightgreen purple)
  datasymbols=(circlefilled trianglefilled)
  datacontrastcolors=(black)
  datalinepatterns=(dashed dot); 
vbar AgeGroup/response=Obsvd&dep stat=mean limits=upper limitstat=stddev limitattrs=(color=black)
  group=Sex groupdisplay=cluster clusterwidth=0.7;
yaxis min=350;
run;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf lsm lsmdiff;

ods select none;
title "Simple comparison of means; Sample size boys,girls=&ssizeb,&ssizeg"; 
title2 "&dep true mean boys,girls=&meanb,&meang";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class Sex AgeGroup;
model Obsvd&dep=Sex AgeGroup Sex*AgeGroup/noint ddfm=sat solution cl alpha=&alpha
  residual outp=pred alphap=&alpha;
repeated/group=Sex*AgeGroup;
estimate "Girls-boys" Sex -1 1/cl alpha=&alpha;
*you can try other estimate statements to compare with lsmeans and lsmean diffs;
lsmeans Sex AgeGroup Sex*AgeGroup/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
*ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title3 "Class levels";
proc print data=clev;
run;

title3 "Covparms (random effects)";
proc print data=cov;
run;

data cov1;
set cov;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title3 "Covparms (random effects) as SD";
proc print data=cov1;
var  CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.0; 
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;

title3 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Fixed-effects solution";
proc print data=solf;
format Estimate StdErr Lower Upper 5.&deceff;
run;

title3 "Least-squares means";
proc print data=lsm;
format Estimate StdErr Lower Upper 5.&deceff;
run;

title3 "Least-squares mean differences";
proc print data=lsmdiff;
format Estimate StdErr Lower Upper 5.&deceff;
run;


title3 "Predicteds and residuals, first 10 observations";
proc print data=pred(obs=10);
run;

proc sort data=pred;
by descending Sex AgeGroup;

data pred1;
set pred;
SexAge=trim(Sex)||" "||AgeGroup;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred1; *uniform=all;
styleattrs  
  datacolors=(red orange yellow lightgreen lightskyblue purple lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10)
  group=SexAge;
run;
ods graphics / reset;

title3 "Outliers (Standardized residual >&tvalue)";
title4 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep Age 5.&decpred StudentResid 5.1;
run;









