/*
  Modified 11b Time series 3x2 trials.sas by having only one testing occasion, with
a repetition of each test in each of the three trials. An acute intervention is
administered after the two tests of the first trial. I have assumed that there is 
extra random error on each of the tests in Trial 1 due to familiarization, and that
there are changes in the mean with extra errors on Trials 2 and 3, to simulate an 
intervention with individual responses. Part of the individual responses are shared 
between Trials 2 and 3. Part of the shared individual responses is correlated with 
the subject's true ability (true &dep) via a value for covarfactor, which is the 
slope of the regression line for individual responses vs ability. Sample values for 
covarfactor are derived via the covariance between ability and individual responses,
and from the slope of the line of individual responses vs ability derived from the 
random-effect solution.
  The fixed effects allow for different means for the two tests (Rep=1 and 2) in 
each trial, but I haven't simulated different means. The random effects allow for 
the residual error to be different in the three trials (but I have simulated extra 
residual error only in the first trial).
  Modify the program as shown in the Proc Mixed step to use the repeated statement.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
%let dep=Speed; *the dependent variable, sprint speed in m/s;
%let mean=160; *true grand mean of dependent, back transforms to exp(300/100)=5.0;
%let sd=10; *true between-subject SD;
%let tem=2; *typical error;
%let deltamean12=4; *change in mean Trial2-Trial1;
%let deltamean13=6; *change in mean Trial3-Trial1;

%let extraerror1=1; *extra residual error on Trial1 due to familiarization;
%let extraerror2=1; *extra error on Trial2 due to indiv responses;
%let extraerror3=2; *extra error on Trial3 due to indiv responses;
%let extraerror23=3; *extra error on Trials 2 and 3 due to shared indiv responses;
%let covarfactor=-0.3; *fraction of subject's true SD contributing to shared individual responses;

%let ss=20; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
do ChildID=1 to &ss;
  True&dep=&mean+&sd*rannor(0);
  XtraError23=&extraerror23*rannor(0)+&covarfactor*(True&dep-&mean);
  Trial=1;
  Rep=1;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Rep=2;
  Obsvd&dep=True&dep+&tem*rannor(0)+&extraerror1*rannor(0);
  output;
  Trial=2;
  Rep=1;
  extraerror2=&extraerror2*rannor(0)+XtraError23;
  Obsvd&dep=True&dep+&deltamean12+&tem*rannor(0)+extraerror2;
  output;
  Rep=2;
  Obsvd&dep=True&dep+&deltamean12+&tem*rannor(0)+extraerror2;
  output;
  Trial=3;
  Rep=1;
  extraerror3=&extraerror3*rannor(0)+XtraError23;
  Obsvd&dep=True&dep+&deltamean13+&tem*rannor(0)+extraerror3;
  output;
  Rep=2;
  Obsvd&dep=True&dep+&deltamean13+&tem*rannor(0)+extraerror3;
  output;
  end;

title "The raw data, the first 6 obs";
proc print data=dat1(obs=6);
format True&dep Obsvd&dep 5.1;
run;

title "Simple stats";
proc means maxdec=1;
var True&dep Obsvd&dep;
class Trial;
run;

*generate means and SDs for plotting;
proc sort data=dat1;
by Trial;

proc means noprint data=dat1;
var Obsvd&dep;
by Trial;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat1 means1;

ods graphics / reset width=8cm height=11cm imagemap attrpriority=none;
title "Line diagram of individual values, and means and SDs";
proc sgplot data=datplot noautolegend;
*styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
series x=Trial y=Obsvd&dep/lineattrs=(pattern=solid color=grey) group=ChildID;
series x=Trial y=Mean/lineattrs=(pattern=solid color=black);
scatter x=Trial y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markerfillattrs=(color=white) 
  markeroutlineattrs=(color=black) markerattrs=(size=12 symbol=circlefilled);
xaxis values=(1,2,3) offsetmin=0.1 offsetmax=0.1;
yaxis label="&dep" labelpos=top;
run;

data dat2;
set dat1;
XvarTrial1=0; *dummy variable to estimate extra variance on Trial 1;
if Trial=1 then XvarTrial1=1;
XvarTrial2=0; *dummy variable to estimate extra variance on Trial 2;
if Trial=2 then XvarTrial2=1;
XvarTrial3=0; *dummy variable to estimate extra variance on Trial 3;
if Trial=3 then XvarTrial3=1;
XvarTrial23=0; *dummy variable to estimate extra variance shared by Trials 2 and 3;
if Trial=2 or Trial=3 then XvarTrial23=1;

*proc print;run;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Time-series 3 trials+repeats; Sample size=&ss; True mean=&mean; True between SD=&sd; Delta mean12,13=&deltamean12,&deltamean13";
title2 "Typical error=&tem; Extra residual error Trial1=&extraerror1; Extra error Trial2,3=&extraerror2,&extraerror3; Extra error Trial23=&extraerror23";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID Trial Rep;
model Obsvd&dep=Trial Rep Trial*Rep/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
random Int XvarTrial2 XvarTrial3/subject=ChildID type=un s cl alpha=&alpha;
repeated/group=Trial;
parms 100 5 5  5 5 5  10 10 10;
*Star off the above and unstar the repeated below to allow for the most general covparms,
 but without estimation of individual responses. Disregard the analysis of the covparms.;
*repeated Trial*Rep/subject=ChildID type=un;
estimate "Trial 2-1" Trial -1 1 0/cl alpha=&alpha;
estimate "Trial 3-1" Trial -1 0 1/cl alpha=&alpha;
estimate "";
estimate "Rep 2-1" Rep -1 1/cl alpha=&alpha;
lsmeans Trial Rep/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title3 "Class levels";
proc print data=clev;
run;

title3 "Covparms (random effects)";
title4 "Check for proper estimation, especially the Residual";
proc print data=cov;
run;


data cov1 covboot;
length CovParm $ 12;
set cov;
DegFree=2*ZValue**2; *Degrees of freedom;
retain LagEst LagSE;
Est=Estimate; SE=StdErr;
if covparm="UN(1,1)" then do;
  covparm="ChildID";
  LagEst=Estimate;
  LagSE=StdErr;
  end;
if covparm="UN(2,1)" then do;
  covparm="CovFactor2";
  Estimate=Est/LagEst;
  Lower=.; Upper=.;
  do i=1 to 5000; *generate parametric bootstrap samples;
    EstimateBoot=(Est+rannor(0)*SE)/(LagEst+rannor(0)*LagSE);
    if Estimate ne 0 then output covboot;
    end;
  *StdErr=(abs(LagEst)*StdErr-abs(Est)*LagSE)/LagEst**2;
  *Lower=Estimate-tinv(1-&alpha/2,DegFree)*Stderr;
  *Upper=Estimate+tinv(1-&alpha/2,DegFree)*Stderr;
  end;
if covparm="UN(2,2)" then covparm="IndivResp2";
if covparm="UN(3,1)" then do;
  covparm="CovFactor3";
  Estimate=Est/LagEst;
  Lower=.; Upper=.;
  do i=1 to 5000; *generate parametric bootstrap samples;
    EstimateBoot=(Est+rannor(0)*SE)/(LagEst+rannor(0)*LagSE);
    if Estimate ne 0 then output covboot;
    end;
  *StdErr=(abs(LagEst)*StdErr-abs(Est)*LagSE)/LagEst**2;
  *Lower=Estimate-tinv(1-&alpha/2,DegFree)*Stderr;
  *Upper=Estimate+tinv(1-&alpha/2,DegFree)*Stderr;
  end;
if covparm="UN(3,2)" then covparm="IndivResp23";
if covparm="UN(3,3)" then covparm="IndivResp3";
array a Estimate Lower Upper; 
if index(CovParm,"CovFactor")=0 then do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statemnt and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
if Estimate ne 0 then output cov1;
run;

title4 "Covparm variances as SD; covariances as slopes";
title5 "True indiv resp Trials 2,3 (IndivResp2,IndivResp3)=sqrt(&extraerror2^2+&extraerror23^2+[&covarfactor*&sd]^2), sqrt(&extraerror3^2+&extraerror23^2+[&covarfactor*&sd]^2)";
title6 "CovFactor is the effect of true ability on individual response as a slope; true values both=&covarfactor";
title7 "True shared indiv resp (IndivResp23)=sqrt(&extraerror23^2+[&covarfactor*&sd]^2)";
proc print data=cov1;
var CovParm Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.1; 
run;

*make macro variables for 5th and 95th percentiles;
data _null_;
call symput('pctlow',&alpha*100/2);
call symput('pctupp',100-&alpha*100/2);

proc univariate noprint data=covboot;
var EstimateBoot Estimate;
output out=covboot1(drop=d) mean=d Estimate median=EstimateBoot pctlpre=X pctlpts=&pctlow,&pctupp
 pctlname=Lower Upper;
by CovParm;
run;

data covboot2;
set covboot1(rename=(XLower=Lower XUpper=Upper));
CLpmApprox=(Upper-Lower)/2;

data cov1a;
set cov1;
if index(CovParm,"CovFactor");

data cov2;
merge cov1a covboot2;
by CovParm;

title5 "CovFactor with an extra decimal place, and bootstrapped estimate and confidence limits";
title6 "CovFactor is the effect of true ability on individual response as a slope; true values both=&covarfactor";
proc print data=cov2;
var CovParm Estimate EstimateBoot CLpmApprox Lower Upper Alpha DegFree;
format Estimate EstimateBoot Lower Upper CLpmApprox 5.2 CLtd 5.2 DegFree 5.1; 
run;


title4 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title4 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title4 "Least-squares means";
proc print data=lsm1;
var Effect Trial Rep Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title4 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Trial _Trial Rep _Rep Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title4 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Trial;
run;
ods graphics / reset;

title4 "Outliers (Standardized residual >&tvalue)";
title5 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title4 "Random-effects solution, first 10 observations";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
run;


*plot individual responses vs ability;
data abil;
set solr;
if Effect="Intercept";
if Estimate ne 0;
keep ChildID Estimate;

data indresp;
set solr;
if Effect="XvarTrial2" or Effect="XvarTrial3";
if Estimate ne 0;
keep ChildID Effect Estimate;

data abilindresp;
merge abil(rename=(Estimate=Ability)) indresp(rename=(Estimate=IndivResp));
by ChildID;
if Effect ne ""; *delete control subjects;

title4 "Relative ability and individual responses from solution for random effects";
title5 "First 10 observations";
proc print data=abilindresp(obs=10);
format Ability IndivResp 5.&deceff;
run;

proc sort;
by Effect;

ods graphics / reset width=13cm height=13cm imagemap attrpriority=none;
title4 "Scatterplot of individual responses vs relative ability";
proc sgplot data=abilindresp;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(dash solid dot);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
refline 0/axis=x lineattrs=(pattern=dot thickness=1 color=black);
reg x=Ability y=IndivResp/nomarkers lineattrs=(thickness=1) group=Effect name="lines";
scatter x=ability y=IndivResp/filledoutlinedmarkers markerattrs=(size=12) group=Effect name="points";
yaxis label="Relative individual response" labelpos=top;
xaxis label="Relative ability";
keylegend "points" "lines";
run;

data est;

ods select none;
title4 "Slope of individual responses vs relative ability";
title5 "True slopes both=&covarfactor; Uncertainty provided by the regression is not shown, because too unrealistically precise";
proc mixed data=abilindresp covtest cl alpha=&alpha nobound;
model IndivResp=Ability/ddfm=sat solution cl residual alpha=&alpha residual outp=pred alphap=&alpha;
estimate "Slope" Ability 1/cl alpha=&alpha;
ods output estimates=est;
by Effect;
run;
ods select all;

*data step to stop getting an error when est is empty;
data est1;
set est;
keep Effect Label Estimate;
format Estimate 5.2;

proc print data=est1;
run;









