/*
  This program exemplifies a random slopes and intercepts mixed model.
  The simulated data consist of repeated measurements on &dep of kids over ages
11-15 y. Each kid has their own slope of the linear trend relating age to &dep,
with units of percent change in &dep per year of age.
  The individual slopes are correlated with the subject's true ability (true &dep)
via a value for covarfactor, which is the slope of the regression line for 
individual differences in the slope of age vs ability. The units of covarfactor 
are (percent change in &dep per year of age) per (percent difference in true &dep).
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
%let dep=VO2max; *the dependent variable;
%let mean=380; *true grand mean of dependent, back transforms to exp(380/100)=44.7;
%let sd=6; *true differences between kids;
%let tem=3; *within-kid typical error in &dep;
%let withinslope=1; *within-subject slope, effect of age on &dep percent per year;
%let sdwithinslope=0.5; *individual differences in the within-subject slope;
%let covarfactor=-0.2; *fraction or multiple of subject's true SD contributing to 
     individual differences in the within-subject slope;
%let ss=30; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
do ChildID=1 to &ss;
  True&dep=&mean+&sd*rannor(0);
  WithinSlope=&withinslope+&sdwithinslope*rannor(0)+&covarfactor*(True&dep-&mean);
  do Age=11 to 15;
    Obsvd&dep=True&dep+WithinSlope*(Age-13)+&tem*rannor(0);
    output;
    end;
  end;

title "Raw data, first 12 obs";
proc print data=dat1(obs=12);
format True&dep Obsvd&dep 5.1 WithinSlope 5.2;
run;

title "Simple stats";
proc means data=dat1 maxdec=1;
run;

ods graphics / reset width=10cm height=10cm imagemap attrpriority=none;
title "Scatterplot of first 7 subjects";
proc sgplot data=dat1;
styleattrs
  datacolors=(red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
scatter x=Age y=Obsvd&dep/filledoutlinedmarkers markerattrs=(symbol=circlefilled size=12) group=ChildID ;
reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1 pattern=dash) group=ChildID ;
where ChildID<8;
run;


*get mean X for each age;
proc sort data=dat1;
by Age;

proc means noprint data=dat1;
var Obsvd&dep;
output out=means mean=Mean std=SD;
by Age;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data plotboth;
set dat1 means1;

ods graphics / reset width=10cm height=10cm imagemap attrpriority=none;
title "Regression of individual values, and means and SDs";
proc sgplot data=plotboth noautolegend;
reg x=Age y=Obsvd&dep/nomarkers lineattrs=(pattern=solid color=lightgrey) group=ChildID;
reg x=Age y=Mean/nomarkers lineattrs=(pattern=solid color=black);
scatter x=Age y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markeroutlineattrs=(color=black)
  markerattrs=(size=12 symbol=circlefilled) markerfillattrs=(color=white);
*xaxis values=(1,2,3) offsetmin=0.1 offsetmax=0.1;
yaxis label="&dep" labelpos=top;
run;


data dat2;
set dat1;
GrandMean=1; *to allow use of lsmeans to get the grand mean, if you don't rescale as below;

*you need this step to remove the artifactual covariance in the random slopes and intercepts model,
 and for the random Intercept to estimate the true differences between kids at the mean age;
*make sdwithinslope=5 to see a profound effect on the random Intercept and covariance,
 if you don't include this step;
proc standard mean=0 data=dat2 out=dat2;
var Age;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Within-subject regression; Ssize=&ss; &dep true mean=&mean; &dep true SD=&sd";
title2 "Age slope mean=&withinslope; Indiv diffs in slope=&sdwithinslope; Typical error in &dep=&tem";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID GrandMean;
model Obsvd&dep=GrandMean Age/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
random Int Age/subject=ChildID type=un s cl alpha=&alpha;
lsmeans GrandMean/cl alpha=&alpha;
estimate "Within slope (per year)" Age 1/cl alpha=&alpha;
estimate "Effect of 6 years" Age 6/cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
run;
ods select all;

title3 "Class levels";
proc print data=clev;
run;

title3 "Covparms";
proc print data=cov;
run;

/*
data cov1;
length Group $ 10; *in case there is a group= in the random effects;
set cov;
if covparm="UN(1,1)" then covparm="ChildID";
if covparm="UN(2,2)" then covparm="Age";
if StdErr ne .;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title3 "Covparms (random effects) as SD";
title4 "ChildID is the estimate of the &dep true SD";
title5 "Age is the estimate of individual differences in the slope";
title6 "Residual is the estimate of the typical error";
proc print data=cov1;
var CovParm Subject Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&decdep CLtd 5.2 DegFree 5.1; 
run;
*/


data cov1 covboot;
length CovParm Group $ 12; *in case there is a group= in the random effects;
set cov;
DegFree=2*ZValue**2; *Degrees of freedom;
LagEst=lag(Estimate);
LagSE=lag(StdErr);
if covparm="UN(1,1)" then covparm="ChildID";
if covparm="UN(2,2)" then covparm="Age";
if covparm="UN(2,1)" then do;
  covparm="CovFactor";
  Est=Estimate; SE=StdErr;
  Estimate=Est/LagEst;
  Lower=.; Upper=.;
  do i=1 to 5000; *generate parametric bootstrap samples;
    EstimateBoot=(Est+rannor(0)*SE)/(LagEst+rannor(0)*LagSE);
    if Estimate ne 0 then output covboot;
    end;
  *StdErr=(abs(LagEst)*StdErr-abs(Est)*LagSE)/LagEst**2; *this was too approximate;
  *Lower=Estimate-tinv(1-&alpha/2,DegFree)*Stderr;
  *Upper=Estimate+tinv(1-&alpha/2,DegFree)*Stderr;
  end;
array a Estimate Lower Upper; 
if CovParm ne "CovFactor" then do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
if Estimate ne 0 then output cov1;
run;

title3 "Covparms (random effects) as SD";
title4 "ChildID is the estimate of the &dep true SD";
title5 "Age is the estimate of individual differences in the slope; true value=sqrt([&covarfactor*&sd]^2+&sdwithinslope^2)";
title6 "CovFactor is the effect of true ability on individual slope; true value=&covarfactor";
title7 "Residual is the estimate of the typical error";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.1; 
run;


*make macro variables for 5th and 95th percentiles;
data _null_;
call symput('pctlow',&alpha*100/2);
call symput('pctupp',100-&alpha*100/2);

proc univariate noprint data=covboot;
var EstimateBoot Estimate;
output out=covboot1(drop=d) mean=d Estimate median=EstimateBoot pctlpre=X pctlpts=&pctlow,&pctupp
 pctlname=Lower Upper;
by Group CovParm;
run;

data covboot2;
set covboot1(rename=(XLower=Lower XUpper=Upper));
CLpmApprox=(Upper-Lower)/2;

data cov1a;
set cov1;
if index(CovParm,"CovFactor");

data cov2;
merge cov1a covboot2;
by Group CovParm;

title4 "CovFactor with an extra decimal place, and bootstrapped estimate and confidence limits";
title5 "CovFactor is the effect of true ability on individual slope; true value=&covarfactor";
proc print data=cov2;
var CovParm Group Estimate EstimateBoot CLpmApprox Lower Upper Alpha DegFree;
format Estimate EstimateBoot Lower Upper CLpmApprox 5.2 CLtd 5.2 DegFree 5.1; 
where CovParm="CovFactor";
run;

title3 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title3 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title3 "Least-squares means";
proc print data=lsm1;
var Effect GrandMean Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Age;
run;
ods graphics / reset;

title3 "Residuals vs Age, to check for non-linearity";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
reg x=Age y=Resid/degree=1 lineattrs=(thickness=1 color=blue) legendlabel="linear" nomarkers;
reg x=Age y=Resid/degree=2 lineattrs=(thickness=1 color=red) legendlabel="quadratic" nomarkers;
scatter x=Age y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Age;
run;
ods graphics / reset;

title3 "Outliers (Standardized residual >&tvalue)";
title4 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title3 "Random-effects solution, first 10 observations";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
run;


*plot individual diffrences in slope vs ability;
*a plot is not produced if the variance for Age is negative;
data abil;
set solr;
if Effect="Intercept";
if Estimate ne 0 and StdErrPred;
keep ChildID Estimate Treat;

*proc print data=solr;run;

data inddiff;
set solr;
if Effect="Age";
if Estimate ne 0; * and StdErrPred;
keep ChildID Effect Estimate Treat;

*proc print;run;

data abilinddiff;
merge abil(rename=(Estimate=Ability)) inddiff(in=a rename=(Estimate=indivdiff));
by ChildID;
if a;

title4 "Relative ability and individual differences from solution for random effects";
title5 "First 10 observations";
proc print data=abilinddiff(obs=10);
format Ability indivdiff 5.&deceff;
run;

*proc sort;
*by Effect;

ods graphics / reset width=10cm height=12cm imagemap attrpriority=none;
title4 "Scatterplot of individual differences vs relative ability";
proc sgplot data=abilinddiff;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(dash solid dot);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
refline 0/axis=x lineattrs=(pattern=dot thickness=1 color=black);
reg x=Ability y=indivdiff/nomarkers lineattrs=(thickness=1) group=Effect;
scatter x=Ability y=indivdiff/filledoutlinedmarkers markerattrs=(size=12) group=Effect;
yaxis label="Relative individual difference" labelpos=top;
xaxis label="Relative ability";
run;

data est;

ods select none;
title4 "Slope of the above line (individual differences vs relative ability) via simple regression";
title5 "True slope=&covarfactor; Uncertainty provided by the regression is not shown, because too unrealistically precise";
proc mixed data=abilinddiff covtest cl alpha=&alpha nobound;
model indivdiff=Ability/ddfm=sat solution cl residual alpha=&alpha residual outp=pred alphap=&alpha;
estimate "Slope" Ability 1/cl alpha=&alpha;
ods output estimates=est;
run;
ods select all;

*data step to stop getting an error when est is empty;
data est1;
set est;
keep Effect Label Estimate;
format Estimate 5.2;

proc print data=est1;
run;






