/*
  This program represents a modification of the program 12 Within subject modeling.sas. 
In that program the simulated data consisted of repeated measurements on each subject
at different ages, and the range of ages was the same for each subject. Such data 
would be analyzed only for the within-subject (individual) relationships and mean 
within-subject relationship. All subjects had the same mean age, so it was not possible
to consider any between-subject relationship between mean age and the dependent variable.
  But sometimes the covariate has a different range of values for each subject, so the 
means of each subject's covariate differ, and it becomes possible to have a between-
subject relationship between the means and the dependent variable (e.g., performance 
indicators of team-sport athletes predicting performance in matches). 
  Hence, in this simulation, the modifying covariate has a different range of values 
within each subject. Each subject has their own linear trend randomly distributed around
a mean trend across subjects, but the subject means can have a different trend. The data 
therefore illustrate not only within-subject modeling (to estimate the individual and 
the mean of the individual relationships), but also between-subject modeling (giving 
only one relationship).
  With a random effect for subject ID, all effects are within-subject. To get the
between-subject relationship, you have to analyze the subject means. The means are 
calculated and included with the individual values in the mixed model to get the
between-subject trend and uncertainty in the trend. The means are also analyzed 
separately in a simple linear regression to get the between-subject trend and 
uncertainty. Both approaches give identical results when there is a fixed number 
of observations per subject. If the number of observations differs between subjects,
the results differ a little, and the simple regression is the better approach.   
  You will find that the analyzed between-subject slope is attenuated, which happens
when there is "noise" in the predictor that is random and unrelated to the dependent. 
There is a simple formula to adjust for such noise. Here the noise is represented   
by variability in the subject means of the X values, which is related to the dependent
by the within-subject relationship, so there is no simple adjustment. Also, the 
standard error of the estimate is greater than the simulated value, owing to the noise.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
%let dep=Perform; *arbitrary name of dependent variable;
%let MeanX=100; *true grand mean of covariate X;
%let sdX=10; *true between-subject SD of X;
%let mean=400; *true grand mean of &dep;
%let see=6; *standard error of the estimate for predicting true &dep;
%let slope=-1; *between-subject slope for predicting &dep;
%let sdwithinX=3; *within-subject variability in X;
%let tem=5; *typical error in &dep;
%let withinslope=1; *within-subject slope;
%let sdwithinslope=1; *individual differences in within-subject slope;
%let covarfactor=-0.1; *fraction or multiple of subject's true SD contributing to 
     individual differences in the within-subject slope;
%let minreps=1; *minimum number of measurements on each subject;
%let maxreps=6; *maximum number of measurements on each subject;
%let ss=30; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for effects;
%let decpred=1; *decimal places for predictor;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
do SubjectID=1 to &ss;
  TrueX=&meanX+&sdX*rannor(0);
  True&dep=&mean+&slope*(TrueX-&meanX)+&see*rannor(0);
  WithinSlope=&withinslope+&sdwithinslope*rannor(0)+&covarfactor*(True&dep-&mean);
  do Trial=1 to max(&minreps,round(&maxreps*ranuni(0))); *from &minreps to &maxreps measurements per subject;
    WithinX=&sdwithinX*rannor(0);
    ObsvdX=TrueX+WithinX;
    Obsvd&dep=True&dep+WithinSlope*WithinX+&tem*rannor(0);
    output;
    end;
  end;

title "Raw data, first 12 obs";
proc print data=dat1(obs=12);
format TruePerform ObsvdPerform 5.1;
run;

title "Simple stats";
proc means data=dat1 maxdec=1;
run;

title "Number of trials per subject";
proc means noprint;
var Trial;
output out=notrials max=NoOfTrials;
by SubjectID;

proc means maxdec=1 n mean min max;
var NoOfTrials;
run;

ods graphics / reset width=10cm height=10cm imagemap attrpriority=none;
title "Scatterplot of first 7 subjects";
proc sgplot data=dat1;
styleattrs
  datacolors=(red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
scatter x=ObsvdX y=Obsvd&dep/filledoutlinedmarkers markerattrs=(symbol=circlefilled size=12) group=SubjectID ;
reg x=ObsvdX y=Obsvd&dep/nomarkers lineattrs=(thickness=1 pattern=solid color=grey) group=SubjectID ;
where SubjectID<8;
run;

*get mean X for each subject;
proc means noprint data=dat1;
var ObsvdX Obsvd&dep;
output out=means mean=MeanX Mean&dep;
by SubjectID;

title "Scatterplot of means";
proc sgplot data=means noautolegend;
reg x=MeanX y=Mean&dep/filledoutlinedmarkers markerattrs=(symbol=circlefilled color=black size=12)
  markerfillattrs=(color=lightgrey) lineattrs=(thickness=1 pattern=solid color=black);
run;

data both;
set dat1 means;

title "Combined scatterplot";
proc sgplot data=both noautolegend;
styleattrs
  datacolors=(red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
reg x=ObsvdX y=Obsvd&dep/nomarkers lineattrs=(thickness=1 pattern=solid color=grey) group=SubjectID ;
*scatter x=ObsvdX y=Obsvd&dep/filledoutlinedmarkers markerattrs=(symbol=circlefilled size=12) group=SubjectID ;
reg x=MeanX y=Mean&dep/filledoutlinedmarkers markerattrs=(symbol=circlefilled color=black size=12)
  markerfillattrs=(color=white) lineattrs=(thickness=1 pattern=solid color=black);
xaxis label="ObsvdX and MeanX";
yaxis label="Obsvd&dep and Mean&dep" labelpos=top;
run;
ods graphics/reset;

data dat2;
set dat1;
GrandMean=1; *to allow use of lsmeans to get the grand mean, if you don't rescale as above;

*you need this step to remove contamination of the within-subject slopes by the between-subject slope,
 and to remove the artifactual covariance in the random slopes and intercepts model,
 and for the random Intercept to estimate the differences between subjects at their mean ObsvdX;
proc standard mean=0 out=dat3;
var ObsvdX;
by SubjectID;

/*
*include this step, if you want to evaluate 2 within-subject SD of ObsvdX;
proc standard mean=0 std=0.5 data=dat3 out=dat3; *note: overwrites dat3;
var ObsvdX;
*/
*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Within-subject regression; Ssize=&ss; True between SD in X=&sdX; True between slope=&slope; SE estimate=&see";
title2 "Within variability in X=&sdwithinX; Within slope=&withinslope; Indiv diffs in slope=&sdwithinslope; Typical error in &dep=&tem";
proc mixed data=dat3 covtest cl alpha=&alpha nobound;
class SubjectID Trial GrandMean;
model Obsvd&dep=GrandMean ObsvdX/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
random Int ObsvdX/subject=SubjectID type=un s cl alpha=&alpha;
lsmeans GrandMean/cl alpha=&alpha;
estimate "ObsvdX (within-subject slope)" ObsvdX 1/cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
run;
ods select all;

title3 "Class levels";
proc print data=clev;
run;

title3 "Covparms";
proc print data=cov;
run;

data cov1 covboot;
length CovParm Group $ 12; *in case there is a group= in the random effects;
set cov;
DegFree=2*ZValue**2; *Degrees of freedom;
LagEst=lag(Estimate);
LagSE=lag(StdErr);
if covparm="UN(1,1)" then covparm="SubjectID";
if covparm="UN(2,2)" then covparm="ObsvdX";
if covparm="UN(2,1)" then do;
  covparm="CovFactor";
  Est=Estimate; SE=StdErr;
  Estimate=Est/LagEst;
  Lower=.; Upper=.;
  do i=1 to 5000; *generate parametric bootstrap samples;
    EstimateBoot=(Est+rannor(0)*SE)/(LagEst+rannor(0)*LagSE);
    if Estimate ne 0 then output covboot;
    end;
  *StdErr=(abs(LagEst)*StdErr-abs(Est)*LagSE)/LagEst**2; *this was too approximate;
  *Lower=Estimate-tinv(1-&alpha/2,DegFree)*Stderr;
  *Upper=Estimate+tinv(1-&alpha/2,DegFree)*Stderr;
  end;
array a Estimate Lower Upper; 
if CovParm ne "CovFactor" then do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
if Estimate ne 0 then output cov1;
run;

title3 "Covparms (random effects) as SD";
title4 "SubjectID is the estimate of the &dep true SD";
title5 "ObsvdX is the estimate of individual differences in the slope; true value=sqrt([&covarfactor*&sd]^2+&sdwithinslope^2)";
title6 "CovFactor is the effect of true ability on individual slope; true value=&covarfactor";
title7 "Residual is the estimate of the typical error";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.1; 
run;

*make macro variables for 5th and 95th percentiles;
data _null_;
call symput('pctlow',&alpha*100/2);
call symput('pctupp',100-&alpha*100/2);

proc univariate noprint data=covboot;
var EstimateBoot Estimate;
output out=covboot1(drop=d) mean=d Estimate median=EstimateBoot pctlpre=X pctlpts=&pctlow,&pctupp
 pctlname=Lower Upper;
by Group CovParm;
run;

data covboot2;
set covboot1(rename=(XLower=Lower XUpper=Upper));
CLpmApprox=(Upper-Lower)/2;

data cov1a;
set cov1;
if index(CovParm,"CovFactor");

data cov2;
merge cov1a covboot2;
by Group CovParm;

title4 "CovFactor with an extra decimal place, and bootstrapped estimate and confidence limits";
title5 "CovFactor is the effect of true ability on individual slope; true value=&covarfactor";
proc print data=cov2;
var CovParm Group Estimate EstimateBoot CLpmApprox Lower Upper Alpha DegFree;
format Estimate EstimateBoot Lower Upper CLpmApprox 5.2 CLtd 5.2 DegFree 5.1; 
where CovParm="CovFactor";
run;


title3 "Fixed-effects solution; True grand mean=&mean";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title3 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title3 "Least-squares means; True grand mean=&mean";
proc print data=lsm1;
var Effect Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10 symbol=circlefilled) markerfillattrs=(color=lightgrey);
run;
ods graphics / reset;

title3 "Residuals vs ObsvdX, to check for non-linearity";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
reg x=ObsvdX y=Resid/degree=1 lineattrs=(thickness=1 color=blue) legendlabel="linear" nomarkers;
reg x=ObsvdX y=Resid/degree=2 lineattrs=(thickness=1 color=red) legendlabel="quadratic" nomarkers;
scatter x=ObsvdX y=Resid/filledoutlinedmarkers markerattrs=(size=10 symbol=circlefilled) markerfillattrs=(color=lightgrey);;
run;
ods graphics / reset;

title3 "Outliers (Standardized residual >&tvalue)";
title4 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title3 "Random-effects solution, first 10 observations";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
run;


*plot individual diffrences in slope vs ability;
*a plot is not produced if the variance for ObsvdX is negative;
data abil;
set solr;
if Effect="Intercept";
if Estimate ne 0 and StdErrPred;
keep SubjectID Estimate Treat;

*proc print data=solr;run;

data inddiff;
set solr;
if Effect="ObsvdX";
if Estimate ne 0; * and StdErrPred;
keep SubjectID Effect Estimate Treat;

*proc print;run;

data abilinddiff;
merge abil(rename=(Estimate=Ability)) inddiff(in=a rename=(Estimate=indivdiff));
by SubjectID;
if a;

title3 "Relative ability and individual differences from solution for random effects";
title4 "First 10 observations";
proc print data=abilinddiff(obs=10);
format Ability indivdiff 5.&deceff;
run;

*proc sort;
*by Effect;

ods graphics / reset width=10cm height=12cm imagemap attrpriority=none;
title3 "Scatterplot of individual differences vs relative ability";
proc sgplot data=abilinddiff;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(dash solid dot);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
refline 0/axis=x lineattrs=(pattern=dot thickness=1 color=black);
reg x=Ability y=indivdiff/nomarkers lineattrs=(thickness=1) group=Effect;
scatter x=Ability y=indivdiff/filledoutlinedmarkers markerattrs=(size=12) group=Effect;
yaxis label="Relative individual difference" labelpos=top;
xaxis label="Relative ability";
run;

data est;

ods select none;
title3 "Slope of the above line (individual differences vs relative ability) via simple regression";
title4 "True slope=&covarfactor; Uncertainty provided by the regression is not shown, because too unrealistically precise";
proc mixed data=abilinddiff covtest cl alpha=&alpha nobound;
model indivdiff=Ability/ddfm=sat solution cl residual alpha=&alpha residual outp=pred alphap=&alpha;
estimate "Slope" Ability 1/cl alpha=&alpha;
ods output estimates=est;
run;
ods select all;

*data step to stop getting an error when est is empty;
data est1;
set est;
keep Effect Label Estimate;
format Estimate 5.2;

proc print data=est1;
run;


*simple linear regression of the means;
*if you want the fixed-effect intercept to be the grand mean, you need this step;
proc standard data=means mean=0 out=means1;
*proc standard mean=0 std=0.5 data=means out=means1; *use this line instead, to evaluate 2 SD of MeanX;
var MeanX;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov cov1 est pred pred1 solf lsm lsmdiff; *solr removed, because used below;

ods select none;
title "Simple linear between-subject regression of means using raw mean &dep";
title2 "Ssize=&ss; True between SD in X=&sdX; True between slope=&slope; SE estimate=&see";
proc mixed data=means1 covtest cl alpha=&alpha nobound;
model Mean&dep=MeanX/ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
estimate "Intercept" Int 1/cl alpha=&alpha;
estimate "MeanX (between-subject slope)" MeanX 1/cl alpha=&alpha;
*ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
*ods output solutionr=solr;
ods output estimates=est;
run;
ods select all;

title3 "Covparms (random effects)";
proc print data=cov;
run;

data cov1;
length Group $ 10; *in case there is no group= in the random effects;
set cov;
if StdErr ne .;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title3 "Covparms (random effects) as SD";
title4 "The residual is the estimate of the SE of the estimate";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&decdep CLtd 5.2  DegFree 5.1; 
run;

title3 "Fixed-effects solution; True Intercept (grand mean)=&mean";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title3 "Fixed-effect estimates; True Intercept (grand mean)=&mean";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
*styleattrs
  datacolors=(red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10 symbol=circlefilled) markerfillattrs=(color=lightgrey);
run;
ods graphics / reset;

title3 "Residuals vs MeanX, to check for non-linearity";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
reg x=MeanX y=Resid/degree=1 lineattrs=(thickness=1 color=blue) legendlabel="linear" nomarkers;
reg x=MeanX y=Resid/degree=2 lineattrs=(thickness=1 color=red) legendlabel="quadratic" nomarkers;
scatter x=MeanX y=Resid/filledoutlinedmarkers markerattrs=(size=10 symbol=circlefilled) markerfillattrs=(color=lightgrey);
run;
ods graphics / reset;

title3 "Outliers (Standardized residual >&tvalue)";
title4 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Mean&dep Pred Resid 5.&decdep StudentResid 5.1;
run;


/*
*simple linear regression of the means;
*using the random-effect solution for SubjectID from within-subject analysis;
*this approach has slightly more attenuation in the slope than using raw means;
data subjid;
set solr;
if Effect="Intercept";
keep SubjectID Estimate;
rename Estimate=RandMean&dep;

data means2;
merge means subjid;
by SubjectID;
run;

*if you want the fixed-effect intercept to be the grand mean, you need this step;
proc standard data=means2 mean=0 out=means3;
*proc standard mean=0 std=0.5 data=means2 out=means3; *use this line instead, to evaluate 2 SD of MeanX;
var MeanX;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov cov1 est pred pred1 solf solr lsm lsmdiff;

ods select none;
title "Simple linear between-subject regression of means using RANDOM-EFFECT SOLUTION for Mean&dep";
title2 "Ssize=&ss; True between SD in X=&sdX; True between slope=&slope; SE estimate=&see";
proc mixed data=means3 covtest cl alpha=&alpha nobound;
model RandMean&dep=MeanX/ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
estimate "Intercept" Int 1/cl alpha=&alpha;
estimate "MeanX (between-subject slope)" MeanX 1/cl alpha=&alpha;
*ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
*ods output solutionr=solr;
ods output estimates=est;
run;
ods select all;

title3 "Covparms (random effects)";
proc print data=cov;
run;

data cov1;
length Group $ 10; *in case there is no group= in the random effects;
set cov;
if StdErr ne .;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title3 "Covparms (random effects) as SD";
title4 "The residual is the estimate of the SE of the estimate";
proc print data=cov1;
var CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&decdep CLtd 5.2  DegFree 5.1; 
run;

title3 "Fixed-effects solution; True Intercept (grand mean)=0";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title3 "Fixed-effect estimates; True Intercept (grand mean)=0";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
*styleattrs
  datacolors=(red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10 symbol=circlefilled) markerfillattrs=(color=lightgrey);
run;
ods graphics / reset;

title3 "Residuals vs MeanX, to check for non-linearity";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
reg x=MeanX y=Resid/degree=1 lineattrs=(thickness=1 color=blue) legendlabel="linear" nomarkers;
reg x=MeanX y=Resid/degree=2 lineattrs=(thickness=1 color=red) legendlabel="quadratic" nomarkers;
scatter x=MeanX y=Resid/filledoutlinedmarkers markerattrs=(size=10 symbol=circlefilled) markerfillattrs=(color=lightgrey);
run;
ods graphics / reset;

title3 "Outliers (Standardized residual >&tvalue)";
title4 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Mean&dep Pred Resid 5.&decdep StudentResid 5.1;
run;
*/








